/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.displaysimulator.ui.controllers;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import org.disperanto.data.TdpCommunicationError;
import org.disperanto.data.TdpNotification;
import org.disperanto.displaysimulator.data.DisplayController;
import org.disperanto.displaysimulator.listeners.IClearNotificationsListener;
import org.disperanto.displaysimulator.listeners.IDisplayControllerListener;
import org.disperanto.displaysimulator.ui.models.NotificationModel;
import org.disperanto.displaysimulator.ui.panels.NotificationPanel;
import org.disperanto.enums.ETdpCommunicationError;
import org.disperanto.enums.ETdpNotification;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * View controller for notificationPanel.
 */
public class NotificationViewController implements ActionListener, IDisplayControllerListener,
        IClearNotificationsListener {
    private static final Logger LOG = LoggerFactory.getLogger(NotificationViewController.class);

    private NotificationModel model;
    private NotificationPanel panel;
    private DisplayController controller;
    private int displayAddress;

    /**
     * Constructor
     * 
     * @param panel Panel that is controlled
     * @param model Model of panel
     * @param controller Display controller
     */
    public NotificationViewController(NotificationPanel panel, NotificationModel model, DisplayController controller) {
        this.model = model;
        this.panel = panel;
        this.controller = controller;
    }

    private void initModel(List<TdpNotification> list) {
        model.setNotifications(list);
    }

    @Override
    public final void actionPerformed(ActionEvent arg0) {
        LOG.debug("Save button pressed");

        if (!validate()) {
            return;
        }

        setNotifications();
    }

    private boolean validate() {
        // no validation

        return true;
    }

    private void setNotifications() {
        final List<TdpNotification> notifications = new ArrayList<TdpNotification>();

        if (panel.getCommunicationErrorCheck().isSelected()) {
            final TdpCommunicationError communicationError = new TdpCommunicationError();
            communicationError.setCommunicationError((ETdpCommunicationError)panel.getCommunicationErrorField()
                    .getSelectedItem());
            notifications.add(communicationError);
        }
        if (panel.getSoftwareFailureCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.SOFTWARE_FAILURE));
        }
        if (panel.getHardwareFailureCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.HARDWARE_FAILURE));
        }
        if (panel.getColdRestartCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.COLD_RESTART));
        }
        if (panel.getWarmRestartCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.WARM_RESTART));
        }
        if (panel.getCommunicationTimeoutCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.COMMUNICATION_TIMEOUT));
        }
        if (panel.getNonCriticalDisplayDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.NON_CRITICAL_DISPLAY_DEFECT));
        }
        if (panel.getCriticalDisplayDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.CRITICAL_DISPLAY_DEFECT));
        }
        if (panel.getIntrusionCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.INTRUSION));
        }
        if (panel.getExternalLightingDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.EXTERNAL_LIGHTING_DEFECT));
        }
        if (panel.getHeatingDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.HEATING_DEFECT));
        }
        if (panel.getCoolingDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.COOLING_DEFECT));
        }
        if (panel.getTemperatureLowCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.TEMPERATURE_LOW));
        }
        if (panel.getTemperatureHighCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.TEMPERATURE_HIGH));
        }
        if (panel.getLuminanceSensorDefectCheck().isSelected()) {
            notifications.add(new TdpNotification(ETdpNotification.LUMINANCE_SENSOR_DEFECT));
        }

        model.setNotifications(notifications);
        
        // copy object to display
        controller.getDisplay(displayAddress).setNotifications(notifications);
    }

    @Override
    public final void displayChanged(int activeDisplay) {
        LOG.debug("Notifications are set");
        displayAddress = activeDisplay;

        // copy object to model
        initModel(new ArrayList<TdpNotification>(controller.getDisplay(displayAddress).getNotifications()));
    }

    @Override
    public final void clear(int displayAddress) {
        model.setNotifications(new ArrayList<TdpNotification>());
    }
}

