/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.displaysimulator.ui.models;

import java.util.ArrayList;
import java.util.List;

import org.disperanto.data.TdpDisplayProperties;
import org.disperanto.displaysimulator.listeners.IPropertiesListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Model for panel with display properties.
 */
public class PropertiesModel {
    private static final Logger LOG = LoggerFactory.getLogger(PropertiesModel.class);

    private int displayAddress;
    private TdpDisplayProperties properties = new TdpDisplayProperties();
    private List<IPropertiesListener> listeners = new ArrayList<IPropertiesListener>();

    /**
     * Constructor
     */
    public PropertiesModel() {

    }

    /**
     * @return the displayAddress
     */
    public final int getDisplayAddress() {
        return displayAddress;
    }

    /**
     * @return the properties
     */
    public final TdpDisplayProperties getProperties() {
        return properties;
    }

    /**
     * @param properties the properties to set
     */
    public final void setProperties(int displayAddress, TdpDisplayProperties properties) {
        LOG.debug("Properties are set");
        this.displayAddress = displayAddress;
        this.properties = properties;
        
        fireChange();
    }

    /**
     * @param listener Listener of changes in display properties
     */
    public final void addListener(IPropertiesListener listener) {
        listeners.add(listener);
    }

    /**
     * fires notification for listeners.
     */
    public final void fireChange() {
        LOG.debug("Fire change");
        for (IPropertiesListener listener : listeners) {
            listener.propertiesChanged(displayAddress, properties);
        }
    }

}
