/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this
   list of conditions and the following disclaimer in the documentation and/or
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 */
package org.disperanto.displaysimulator.ui.panels;

import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.disperanto.communication.IStartableConnection;
import org.disperanto.communication.TcpReceiver;
import org.disperanto.communication.UdpReceiver;
import org.disperanto.displaysimulator.listeners.IDisplayControllerListener;
import org.disperanto.displaysimulator.ui.components.NumericTextField;
import org.disperanto.enums.EIpConnectionType;

import net.miginfocom.swing.MigLayout;

/**
 * Panel for changing connection settings and choosing display.
 */
public class DisplayControllerPanel extends JPanel implements IDisplayControllerListener {

    // components
    private JComboBox<EIpConnectionType> connectionTypeBox;
    private NumericTextField receivePort;
    private NumericTextField timeout;

    private JButton applyButton;

    /**
     * Constructor
     */
    public DisplayControllerPanel() {
        initUI();
    }

    /**
     * Add listener for panel actions
     *
     * @param listener Action listener
     */
    public final void addListener(ActionListener listener) {
        applyButton.addActionListener(listener);
    }

    private void initUI() {
        setLayout(new MigLayout());
        setBorder(BorderFactory.createTitledBorder("Display controller"));


        add(new JLabel("Connection type:"));
        Vector<EIpConnectionType> values = new Vector<EIpConnectionType>(EIpConnectionType.getEnabled());
        connectionTypeBox = new JComboBox<EIpConnectionType>(values);
        add(connectionTypeBox, "gapright 10");
        connectionTypeBox.setSelectedItem(EIpConnectionType.TCP);

        receivePort = new NumericTextField(3456, 1025, 0xFFFF);
        add(new JLabel("Receive port:"));
        add(receivePort, "gapright 10");

        JLabel timeoutLabel = new JLabel("Connection timeout:");
        add(timeoutLabel);
        timeout = new NumericTextField(5000);
        add(timeout, "gapright 10");

        applyButton = new JButton("Apply");
        add(applyButton);
    }

    @Override
    public final void displayChanged(int address) {
    }

    /**
     * @return the connectionType
     */
    public final EIpConnectionType getConnectionType() {
        return (EIpConnectionType)connectionTypeBox.getSelectedItem();
    }

    /**
     * @return the receivePort
     */
    public final NumericTextField getReceivePort() {
        return receivePort;
    }

    /**
     * @return the timeout
     */
    public final NumericTextField getTimeout() {
        return timeout;
    }

    public IStartableConnection getConnection() {
        EIpConnectionType type = (EIpConnectionType)connectionTypeBox.getSelectedItem();
        switch (type) {
        case UDP:
            return new UdpReceiver(safeLongToInt(receivePort.getValue()), safeLongToInt(timeout.getValue()));
        case TCP:
            return new TcpReceiver(safeLongToInt(receivePort.getValue()), safeLongToInt(timeout.getValue()));
        default:
            assert false; // This is impossible, all cases covered.
            return null;
        }
    }

    private static int safeLongToInt(long value) {
        if (value < Integer.MIN_VALUE || value > Integer.MAX_VALUE) {
            throw new IllegalArgumentException(value + " cannot be cast to int without changing its value.");
        }
        return (int)value;
    }
}
