/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this
   list of conditions and the following disclaimer in the documentation and/or
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 */
package org.disperanto.displaysimulator.ui.panels;

import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.miginfocom.swing.MigLayout;

import org.disperanto.data.TdpCommunicationError;
import org.disperanto.data.TdpNotification;
import org.disperanto.displaysimulator.listeners.INotificationListener;
import org.disperanto.enums.ETdpCommunicationError;
import org.disperanto.enums.ETdpNotification;

/**
 * Panel for showing and changing which notifications are send for a display.
 */
public class NotificationPanel extends JPanel implements INotificationListener {
    // private static final Logger LOG = LoggerFactory.getLogger(NotificationPanel.class);

    // components
    private JCheckBox communicationErrorCheck;
    private JComboBox<ETdpCommunicationError> communicationErrorField;
    private JCheckBox softwareFailureCheck;
    private JCheckBox hardwareFailureCheck;
    private JCheckBox coldRestartCheck;
    private JCheckBox warmRestartCheck;
    private JCheckBox communicationTimeoutCheck;
    private JCheckBox nonCriticalDisplayDefectCheck;
    private JCheckBox criticalDisplayDefectCheck;
    private JCheckBox intrusionCheck;
    private JCheckBox externalLightingDefectCheck;
    private JCheckBox heatingDefectCheck;
    private JCheckBox coolingDefectCheck;
    private JCheckBox temperatureLowCheck;
    private JCheckBox temperatureHighCheck;
    private JCheckBox luminanceSensorDefectCheck;

    private JButton saveButton;

    /**
     * Constructor
     */
    public NotificationPanel() {
        initUI();
    }

    /**
     * Add listener for panel actions
     *
     * @param listener Action listener
     */
    public final void addListener(ActionListener listener) {
        saveButton.addActionListener(listener);
    }

    private void initUI() {
        setLayout(new MigLayout());
        setBorder(BorderFactory.createTitledBorder("Notifications"));

        communicationErrorCheck = new JCheckBox();
        communicationErrorField = new JComboBox<ETdpCommunicationError>(ETdpCommunicationError.values());
        softwareFailureCheck = new JCheckBox();
        hardwareFailureCheck = new JCheckBox();
        coldRestartCheck = new JCheckBox();
        warmRestartCheck = new JCheckBox();
        communicationTimeoutCheck = new JCheckBox();
        nonCriticalDisplayDefectCheck = new JCheckBox();
        criticalDisplayDefectCheck = new JCheckBox();
        intrusionCheck = new JCheckBox();
        externalLightingDefectCheck = new JCheckBox();
        heatingDefectCheck = new JCheckBox();
        coolingDefectCheck = new JCheckBox();
        temperatureLowCheck = new JCheckBox();
        temperatureHighCheck = new JCheckBox();
        luminanceSensorDefectCheck = new JCheckBox();

        add(new JLabel("Communication error:"));
        add(communicationErrorCheck, "");
        add(communicationErrorField, "wrap");

        add(new JLabel("Software Failure:"));
        add(softwareFailureCheck, "wrap");

        add(new JLabel("Hardware Failure:"));
        add(hardwareFailureCheck, "wrap");

        add(new JLabel("Cold restart:"));
        add(coldRestartCheck, "wrap");

        add(new JLabel("Warm restart:"));
        add(warmRestartCheck, "wrap");

        add(new JLabel("Communication timeout:"));
        add(communicationTimeoutCheck, "wrap");

        add(new JLabel("Non critical display defect:"));
        add(nonCriticalDisplayDefectCheck, "wrap");

        add(new JLabel("Critical display defect:"));
        add(criticalDisplayDefectCheck, "wrap");

        add(new JLabel("Intrusion:"));
        add(intrusionCheck, "wrap");

        add(new JLabel("Heating defect:"));
        add(heatingDefectCheck, "wrap");

        add(new JLabel("Cooling defect:"));
        add(coolingDefectCheck, "wrap");

        add(new JLabel("Temperature low:"));
        add(temperatureLowCheck, "wrap");

        add(new JLabel("Temperature high:"));
        add(temperatureHighCheck, "wrap");

        add(new JLabel("Luminance sensor defect:"));
        add(luminanceSensorDefectCheck, "wrap");

        saveButton = new JButton("Apply");
        add(new JLabel());
        add(saveButton, "wrap");
    }

    @Override
    public final void notificationsChanged(int displayAddress, List<TdpNotification> notifications) {
        communicationErrorCheck.setSelected(false);
        softwareFailureCheck.setSelected(false);
        hardwareFailureCheck.setSelected(false);
        coldRestartCheck.setSelected(false);
        warmRestartCheck.setSelected(false);
        communicationTimeoutCheck.setSelected(false);
        nonCriticalDisplayDefectCheck.setSelected(false);
        criticalDisplayDefectCheck.setSelected(false);
        intrusionCheck.setSelected(false);
        externalLightingDefectCheck.setSelected(false);
        heatingDefectCheck.setSelected(false);
        coolingDefectCheck.setSelected(false);
        temperatureLowCheck.setSelected(false);
        temperatureHighCheck.setSelected(false);
        luminanceSensorDefectCheck.setSelected(false);

        for (TdpNotification notification : notifications) {
            if (notification.getTag() == ETdpNotification.COMMUNICATION_ERROR) {
                final TdpCommunicationError commmunicationError = (TdpCommunicationError)notification;
                communicationErrorCheck.setSelected(true);
                communicationErrorField.setSelectedItem(commmunicationError.getCommunicationError());
            }
            if (notification.getTag() == ETdpNotification.SOFTWARE_FAILURE) {
                softwareFailureCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.HARDWARE_FAILURE) {
                hardwareFailureCheck.setSelected(true);
            }

            if (notification.getTag() == ETdpNotification.COLD_RESTART) {
                coldRestartCheck.setSelected(true);
            }

            if (notification.getTag() == ETdpNotification.WARM_RESTART) {
                warmRestartCheck.setSelected(true);
            }

            if (notification.getTag() == ETdpNotification.COMMUNICATION_TIMEOUT) {
                communicationTimeoutCheck.setSelected(true);
            }

            if (notification.getTag() == ETdpNotification.NON_CRITICAL_DISPLAY_DEFECT) {
                nonCriticalDisplayDefectCheck.setSelected(true);
            }

            if (notification.getTag() == ETdpNotification.CRITICAL_DISPLAY_DEFECT) {
                criticalDisplayDefectCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.INTRUSION) {
                intrusionCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.EXTERNAL_LIGHTING_DEFECT) {
                externalLightingDefectCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.HEATING_DEFECT) {
                heatingDefectCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.COOLING_DEFECT) {
                coolingDefectCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.TEMPERATURE_LOW) {
                temperatureLowCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.TEMPERATURE_HIGH) {
                temperatureHighCheck.setSelected(true);
            }
            if (notification.getTag() == ETdpNotification.LUMINANCE_SENSOR_DEFECT) {
                luminanceSensorDefectCheck.setSelected(true);
            }
        }
    }

    /**
     * @return the communicationErrorCheck
     */
    public final JCheckBox getCommunicationErrorCheck() {
        return communicationErrorCheck;
    }

    /**
     * @return the communicationErrorField
     */
    public final JComboBox<ETdpCommunicationError> getCommunicationErrorField() {
        return communicationErrorField;
    }

    /**
     * @return the softwareFailureCheck
     */
    public final JCheckBox getSoftwareFailureCheck() {
        return softwareFailureCheck;
    }

    /**
     * @return the hardwareFailureCheck
     */
    public final JCheckBox getHardwareFailureCheck() {
        return hardwareFailureCheck;
    }

    /**
     * @return the coldRestartCheck
     */
    public final JCheckBox getColdRestartCheck() {
        return coldRestartCheck;
    }

    /**
     * @return the warmRestartCheck
     */
    public final JCheckBox getWarmRestartCheck() {
        return warmRestartCheck;
    }

    /**
     * @return the communicationTimeoutCheck
     */
    public final JCheckBox getCommunicationTimeoutCheck() {
        return communicationTimeoutCheck;
    }

    /**
     * @return the nonCriticalDisplayDefectCheck
     */
    public final JCheckBox getNonCriticalDisplayDefectCheck() {
        return nonCriticalDisplayDefectCheck;
    }

    /**
     * @return the criticalDisplayDefectCheck
     */
    public final JCheckBox getCriticalDisplayDefectCheck() {
        return criticalDisplayDefectCheck;
    }

    /**
     * @return the intrusionCheck
     */
    public final JCheckBox getIntrusionCheck() {
        return intrusionCheck;
    }

    /**
     * @return the externalLightingDefectCheck
     */
    public final JCheckBox getExternalLightingDefectCheck() {
        return externalLightingDefectCheck;
    }

    /**
     * @return the heatingDefectCheck
     */
    public final JCheckBox getHeatingDefectCheck() {
        return heatingDefectCheck;
    }

    /**
     * @return the coolingDefectCheck
     */
    public final JCheckBox getCoolingDefectCheck() {
        return coolingDefectCheck;
    }

    /**
     * @return the temperatureLowCheck
     */
    public final JCheckBox getTemperatureLowCheck() {
        return temperatureLowCheck;
    }

    /**
     * @return the temperatureHighCheck
     */
    public final JCheckBox getTemperatureHighCheck() {
        return temperatureHighCheck;
    }

    /**
     * @return the luminanceSensorDefectCheck
     */
    public final JCheckBox getLuminanceSensorDefectCheck() {
        return luminanceSensorDefectCheck;
    }
}
