/*
Copyright (c) 2013-2019, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.displaysimulator.ui.panels;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.disperanto.displaysimulator.listeners.ITextListener;

import net.miginfocom.swing.MigLayout;

/**
 * Panel for showing text from text banks of display.
 */
public final class TextPanel extends JPanel implements ITextListener {

    private JTextArea textLines;
    private JLabel displayEnabledLabel;

    /**
     * Constructor
     */
    public TextPanel() {
        initUI();
    }

    private void initUI() {
        setLayout(new MigLayout("", "[][][grow]", "[fill][]"));

        textLines = new JTextArea();
        textLines.setEditable(false);
        textLines.setLineWrap(false);
        add(new JScrollPane(textLines), "grow, push, wrap, span");

        displayEnabledLabel = new JLabel();
        add(new JLabel("DRIP power state:"), "");
        add(displayEnabledLabel, "wrap");
    }

    /**
     * @return the textLines
     */
    public final JTextArea getTextLines() {
        return textLines;
    }

    @Override
    public void textChanged(String text, boolean isDisplayEnabled) {
        textLines.setText(text);
        displayEnabledLabel.setText(isDisplayEnabled ? "enabled" : "disabled");
    }
}
