/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.displaysimulator.utils;

import java.util.concurrent.TimeUnit;

/**
 * Contains thread related utilities.
 */
public final class ThreadHelper {

    private ThreadHelper() {
    }
    
    /**
     * Sleep without having to try/catch the InterruptedException.
     * 
     * @param milliseconds Milliseconds to sleep.
     */
    public static void sleep(long milliseconds) {
        try {
            Thread.sleep(milliseconds);
        } catch (InterruptedException ex) {
            throw new IllegalStateException("Unexpected InterruptedException");
        }
    }

    /**
     * Sleep without having to try/catch the InterruptedException.
     * 
     * @param duration Duration to sleep.
     * @param unit Unit of duration.
     */
    public static void sleep(long duration, TimeUnit unit) {
        try {
            long ns = TimeUnit.NANOSECONDS.convert(duration, unit);
            
            Thread.sleep(ns / 1000000L, (int)(ns % 1000000L));
        } catch (InterruptedException ex) {
            throw new IllegalStateException("Unexpected InterruptedException");
        }
    }

    /**
     * Join without having to try/catch the InterruptedException.
     * 
     * @param thread Thread to join with.
     * @param milliseconds Milliseconds to wait for join to complete.
     * @return True when join completed successfully.
     */
    public static boolean join(Thread thread, long milliseconds) {
        try {
            thread.join(milliseconds);
            return !thread.isAlive();
        } catch (InterruptedException ex) {
            throw new IllegalStateException("Unexpected InterruptedException");
        }
    }

}
