/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.controllers;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.UnknownHostException;

import org.disperanto.communication.IStartableConnection;
import org.disperanto.simulator.data.LogEvent;
import org.disperanto.simulator.ui.models.IConnectionModel;
import org.disperanto.simulator.ui.models.LoggingModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * View controller for ConnectionPanel
 */
public class ConnectionViewController implements ActionListener {

    private static final Logger LOG = LoggerFactory.getLogger(ConnectionViewController.class);

    private final Server server;
    private final IConnectionModel model;
    private final LoggingModel loggingModel;

    public ConnectionViewController(IConnectionModel model, Server server, LoggingModel lm) {
        this.model = model;
        this.server = server;
        this.loggingModel = lm;
        setConnection();
    }

    @Override
    public final void actionPerformed(ActionEvent arg0) {
        LOG.debug("Apply button pressed");

        setConnection();
    }

    private void setConnection() {
        try {
            final IStartableConnection connection = model.getConnection();
            server.setConnection(connection);
        } catch (RuntimeException e) {
            if (e.getCause() instanceof UnknownHostException) {
                loggingModel.addRow(new LogEvent("Unknown host: " + e.getCause().getMessage()));
            } else {
                loggingModel.addRow(new LogEvent("Unexpected exception: " + e));
                throw e;
            }
        }
    }
}
