/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.controllers;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import org.disperanto.messages.TdpMessage;
import org.disperanto.simulator.ui.panels.ResponseListPanel;


/**
 * View controller for CommandListPanel
 */
public class ResponseListViewController implements TreeSelectionListener {
    private ResponseListPanel panel;

    /**
     * Constructor
     * 
     * @param panel Panel that is controlled
     */
    public ResponseListViewController(ResponseListPanel panel) {
        this.panel = panel;
    }

    @Override
    public final void valueChanged(TreeSelectionEvent arg0) {
        final DefaultMutableTreeNode node = (DefaultMutableTreeNode)panel.getResponseTree()
                .getLastSelectedPathComponent();
        TdpMessage response;

        /* if nothing is selected */
        if (node == null) {
            return;
        }

        /* retrieve the node that was selected */
        if (node.getUserObject() instanceof TdpMessage) {
            response = (TdpMessage)node.getUserObject();
        } else if (node.getUserObject() instanceof Integer) {
            final DefaultMutableTreeNode parent = (DefaultMutableTreeNode)node.getParent();
            response = (TdpMessage)parent.getUserObject();
        } else {
            return;
        }

        panel.getMessageData().setText(response.toReadableString());
        panel.getMessageData().setCaretPosition(0);
    }
}
