/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.ui.components;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;

import org.disperanto.simulator.utils.HotkeyUtils;


/**
 * Button with support for hotkeys.
 * 
 * Use an ampersand (&amp;) to select the hotkey for this button.
 * For example in "&Test", the user can press Alt+T to activate the button.
 */
public class MmButton extends JButton {

    /**
     * Constructor
     * 
     * @param text Text of button
     * @param name Button name
     */
    public MmButton(String text, String name) {
        this(text, name, null);
    }

    /**
     * Constructor
     * 
     * @param text Text of button
     * @param name Button name
     * @param icon Icon on button
     */
    public MmButton(String text, String name, ImageIcon icon) {
        setAction(new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                MmButton.this.actionPerformed(e);
            }
        });
        setName(name);
        setText(text);
        setIcon(icon);
    }

    @Override
    public final void setText(String name) {
        super.setText(HotkeyUtils.stripMnemonic(name));
        if (HotkeyUtils.hasMnemonic(name)) {
            super.setMnemonic(HotkeyUtils.getMnemonic(name));
        }
    }

    /**
     * Action to perform
     * 
     * @param e Action event
     */
    public void actionPerformed(ActionEvent e) {
        // No default implementation
    }
}
