/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.ui.panels;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.disperanto.data.TdpBrightnessTable;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.simulator.ui.models.IBrightnessTableModel;

import net.miginfocom.swing.MigLayout;

/**
 * Panel for brightness table.
 */
public class BrightnessTablePanel extends JPanel implements IBrightnessTableModel {

    private final List<JSpinner> spinners;

    public BrightnessTablePanel() {
        super(new MigLayout());
        spinners = new ArrayList<JSpinner>();
        initGui();
    }

    @Override
    public TdpBrightnessTable getBrightnessTable() {
        TdpBrightnessTable table = new TdpBrightnessTable();

        ArrayList<Integer> values = new ArrayList<Integer>();
        for (JSpinner spinner : spinners) {
            values.add((Integer)spinner.getValue());
        }
        table.setBrightnessTable(values);

        return table;
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.SET_BRIGHTNESS));
    }

    private void setComponentsEnabled(boolean enabled) {
        for (JSpinner spinner : spinners) {
            spinner.setEnabled(enabled);
        }
    }

    private void initGui() {
        setBorder(BorderFactory.createTitledBorder("Brightness table"));
        for (int i = 0; i < 11; i++) {
            spinners.add(new JSpinner(new SpinnerNumberModel(i * 10, 0, 100, 1)));
            add(spinners.get(i));
        }
    }
}
