/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.ui.panels;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.disperanto.enums.ETdpCommand;
import org.disperanto.simulator.ui.models.ICrcListModel;

import net.miginfocom.swing.MigLayout;

/**
 * Panel showing CRC's of image slots.
 */
public class CrcListPanel extends JPanel implements ICrcListModel {

    private final JTextField slotListField;

    public CrcListPanel() {
        super(new MigLayout("", "[grow, align right][110]", ""));
        setBorder(BorderFactory.createTitledBorder("Calculate CRC"));
        add(new JLabel("Display slots: "));
        slotListField = new JTextField("2,3");
        slotListField.setHorizontalAlignment(JTextField.TRAILING);
        add(slotListField, "grow");
    }

    @Override
    public List<Integer> getSlotList() {
        return validateData();
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.CALCULATE_CRC_IMAGES));
    }

    private void setComponentsEnabled(boolean enabled) {
        slotListField.setEnabled(enabled);
    }

    private List<Integer> validateData() {
        List<Integer> slots = new ArrayList<Integer>();

        String[] slotStrings = slotListField.getText().split(",");
        for (String slotString : slotStrings) {

            try {
                slots.add(Integer.parseInt(slotString));
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(this,
                        "Wrong format in CRC slot list, only use comma separated list of numbers", "Wrong format",
                        JOptionPane.ERROR_MESSAGE);
            }
        }

        return slots;
    }
}
