/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this
   list of conditions and the following disclaimer in the documentation and/or
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 */
package org.disperanto.simulator.ui.panels;

import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.miginfocom.swing.MigLayout;

import org.disperanto.data.TdpExternalLighting;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpLightingState;
import org.disperanto.simulator.ui.models.IExternalLightingModel;

/**
 * Panel showing external lighting state.
 */
public class ExternalLightingPanel extends JPanel implements IExternalLightingModel {

    private final JComboBox<ETdpLightingState> stateBox;

    public ExternalLightingPanel() {
        super(new MigLayout("", "[grow, align right][110]", ""));
        setBorder(BorderFactory.createTitledBorder("External lighting settings"));
        add(new JLabel("State: "));
        stateBox = new JComboBox<ETdpLightingState>(ETdpLightingState.values());
        add(stateBox);
    }

    @Override
    public TdpExternalLighting getLightingSettings() {
        TdpExternalLighting lighting = new TdpExternalLighting();
        lighting.setState((ETdpLightingState)stateBox.getSelectedItem());
        return lighting;
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.SET_EXTERNAL_LIGHTING));
    }

    private void setComponentsEnabled(boolean enabled) {
        stateBox.setEnabled(enabled);
    }

}
