/*
Copyright (c) 2013,2014, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this
   list of conditions and the following disclaimer in the documentation and/or
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 */
package org.disperanto.simulator.ui.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import net.miginfocom.swing.MigLayout;

import org.disperanto.data.IImageOperation;
import org.disperanto.data.TdpClearRectangle;
import org.disperanto.data.TdpCopyImage;
import org.disperanto.data.TdpInitializeWorkingMemory;
import org.disperanto.data.TdpLoadImage;
import org.disperanto.data.TdpManipulateMemorySlot;
import org.disperanto.data.TdpStoreImage;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpImageType;
import org.disperanto.simulator.ui.components.NumericTextField;
import org.disperanto.simulator.ui.images.TdpServerSimulatorImages;
import org.disperanto.simulator.ui.models.IManipulateMemorySlotModel;

public class ManipulateMemorySlotPanel extends JPanel implements IManipulateMemorySlotModel {

    private JCheckBox cbInitializeWorkingMemory;
    private NumericTextField initWidth;
    private NumericTextField initHeight;

    private JCheckBox cbClearRectangle;
    private NumericTextField clearLeft;
    private NumericTextField clearTop;
    private NumericTextField clearWidth;
    private NumericTextField clearHeight;

    private JCheckBox cbLoadImage;
    private NumericTextField loadLeft;
    private NumericTextField loadTop;
    private JComboBox<ETdpImageType> loadTypeBox;
    private File loadImage;
    private JButton loadImageButton;

    private JCheckBox cbCopyImage;
    private NumericTextField copyLeft;
    private NumericTextField copyTop;
    private NumericTextField copySlot;

    private JCheckBox cbStoreImage;
    private NumericTextField storeSlot;

    public ManipulateMemorySlotPanel() {
        super(new MigLayout("", "[grow, align right][grow]", "[]"));
        setBorder(BorderFactory.createTitledBorder("Manipulate memory slot"));

        cbInitializeWorkingMemory = new JCheckBox("Initialize working memory:", true);
        initWidth = new NumericTextField(200,  0, 2000);
        initHeight = new NumericTextField(200,  0, 2000);
        add(cbInitializeWorkingMemory, "grow");
        JPanel initializeWorkingMemoryPanel = new JPanel(new MigLayout("", "[grow, align right][110]"));
        add(initializeWorkingMemoryPanel, "wrap");
        initializeWorkingMemoryPanel.add(new JLabel("Width: "), "grow");
        initializeWorkingMemoryPanel.add(initWidth, "grow, wrap");
        initializeWorkingMemoryPanel.add(new JLabel("Height: "), "grow");
        initializeWorkingMemoryPanel.add(initHeight, "grow");

        cbClearRectangle = new JCheckBox("Clear rectangle:", false);
        clearTop = new NumericTextField(0,  0, 2000);
        clearLeft = new NumericTextField(0,  0, 2000);
        clearWidth = new NumericTextField(200,  0, 2000);
        clearHeight = new NumericTextField(200,  0, 2000);
        add(cbClearRectangle, "grow");
        JPanel clearRectanglePanel = new JPanel(new MigLayout("", "[grow, align right][110]"));
        add(clearRectanglePanel, "wrap");
        clearRectanglePanel.add(new JLabel("Top: "), "grow");
        clearRectanglePanel.add(clearTop, "grow, wrap");
        clearRectanglePanel.add(new JLabel("Left: "), "grow");
        clearRectanglePanel.add(clearLeft, "grow, wrap");
        clearRectanglePanel.add(new JLabel("Width: "), "grow");
        clearRectanglePanel.add(clearWidth, "grow, wrap");
        clearRectanglePanel.add(new JLabel("Height: "), "grow");
        clearRectanglePanel.add(clearHeight, "grow, wrap");

        cbLoadImage = new JCheckBox("Load image:", true);
        loadTop = new NumericTextField(0,  0, 2000);
        loadLeft = new NumericTextField(0,  0, 2000);
        loadTypeBox = new JComboBox<ETdpImageType>(ETdpImageType.values());
        loadTypeBox.setSelectedItem(ETdpImageType.PNG);
        loadImageButton = new JButton("select");
        loadImageButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser(loadImage);
                chooser.setFileFilter(new FileFilter() {

                    @Override
                    public String getDescription() {
                        return "Image file (*." + ((ETdpImageType)loadTypeBox.getSelectedItem()).getDescription() + ")";
                    }

                    @Override
                    public boolean accept(File f) {
                        if (f.isDirectory()) {
                            return true;
                        } else {
                            switch ((ETdpImageType)loadTypeBox.getSelectedItem()) {
                            case BMP:
                                return f.getName().toLowerCase().endsWith(".bmp");
                            case PNG:
                                return f.getName().toLowerCase().endsWith(".png");
                            default:
                                return true;
                            }
                        }
                    }
                });
                if (chooser.showOpenDialog(ManipulateMemorySlotPanel.this) == JFileChooser.APPROVE_OPTION) {
                    loadImage = chooser.getSelectedFile();
                }
            }
        });
        add(cbLoadImage, "grow");
        JPanel loadImagePanel = new JPanel(new MigLayout("", "[grow, align right][110]"));
        add(loadImagePanel, "wrap");
        loadImagePanel.add(new JLabel("Top: "), "grow");
        loadImagePanel.add(loadTop, "grow, wrap");
        loadImagePanel.add(new JLabel("Left: "), "grow");
        loadImagePanel.add(loadLeft, "grow, wrap");
        loadImagePanel.add(new JLabel("Type: "), "grow");
        loadImagePanel.add(loadTypeBox, "grow, wrap");
        loadImagePanel.add(new JLabel("Load image: "), "grow");
        loadImagePanel.add(loadImageButton, "grow, wrap");

        cbCopyImage = new JCheckBox("Copy image:", false);
        copyTop = new NumericTextField(0,  0, 2000);
        copyLeft = new NumericTextField(0,  0, 2000);
        copySlot = new NumericTextField(2,  0, 999);
        add(cbCopyImage, "grow");
        JPanel copyImagePanel = new JPanel(new MigLayout("", "[grow, align right][110]"));
        add(copyImagePanel, "wrap");
        copyImagePanel.add(new JLabel("Top: "), "grow");
        copyImagePanel.add(copyTop, "grow, wrap");
        copyImagePanel.add(new JLabel("Left: "), "grow");
        copyImagePanel.add(copyLeft, "grow, wrap");
        copyImagePanel.add(new JLabel("Memory slot: "), "grow");
        copyImagePanel.add(copySlot, "grow, wrap");

        cbStoreImage = new JCheckBox("Store image:", true);
        storeSlot = new NumericTextField(2,  0, 999);
        add(cbStoreImage, "grow");
        JPanel storeImagePanel = new JPanel(new MigLayout("", "[grow, align right][110]"));
        add(storeImagePanel, "wrap");
        storeImagePanel.add(new JLabel("Memory slot: "), "grow");
        storeImagePanel.add(storeSlot, "grow");
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.MANIPULATE_MEMORY_SLOT));
    }

    private void setComponentsEnabled(boolean enabled) {
        cbInitializeWorkingMemory.setEnabled(enabled);
        initWidth.setEnabled(enabled);
        initHeight.setEnabled(enabled);
        cbClearRectangle.setEnabled(enabled);
        clearLeft.setEnabled(enabled);
        clearTop.setEnabled(enabled);
        clearWidth.setEnabled(enabled);
        clearHeight.setEnabled(enabled);
        cbLoadImage.setEnabled(enabled);
        loadLeft.setEnabled(enabled);
        loadTop.setEnabled(enabled);
        loadTypeBox.setEnabled(enabled);
        loadImageButton.setEnabled(enabled);
        cbCopyImage.setEnabled(enabled);
        copyLeft.setEnabled(enabled);
        copyTop.setEnabled(enabled);
        copySlot.setEnabled(enabled);
        cbStoreImage.setEnabled(enabled);
        storeSlot.setEnabled(enabled);
    }

    @Override
    public TdpManipulateMemorySlot getManipulateMemorySlotSettings() {
        List<IImageOperation> operations = new ArrayList<IImageOperation>();

        if (cbInitializeWorkingMemory.isSelected()) {
            // clear working memory with size of display
            TdpInitializeWorkingMemory tiwm = new TdpInitializeWorkingMemory();
            tiwm.setHeight((int)initHeight.getValue());
            tiwm.setWidth((int)initWidth.getValue());
            operations.add(tiwm);
        }
        if (cbClearRectangle.isSelected()) {
            TdpClearRectangle tcr = new TdpClearRectangle();
            tcr.setLeft((int)clearLeft.getValue());
            tcr.setTop((int)clearTop.getValue());
            tcr.setHeight((int)clearHeight.getValue());
            tcr.setWidth((int)clearWidth.getValue());
            operations.add(tcr);
        }
        if (cbLoadImage.isSelected()) {
            TdpLoadImage tli = new TdpLoadImage();
            tli.setTop((int)loadTop.getValue());
            tli.setLeft((int)loadLeft.getValue());
            tli.setImageType((ETdpImageType)loadTypeBox.getSelectedItem());
            try {
                tli.setImage(loadImage == null ? TdpServerSimulatorImages.getDefaultImageForLoadImageCommand() :
                    TdpServerSimulatorImages.getImageForLoadImageCommand(new FileInputStream(loadImage)));
            } catch (FileNotFoundException e) {
                // loading failed, use default
                tli.setImage(TdpServerSimulatorImages.getDefaultImageForLoadImageCommand());
            }
            operations.add(tli);
        }
        if (cbCopyImage.isSelected()) {
            TdpCopyImage tci = new TdpCopyImage();
            tci.setTop((int)copyTop.getValue());
            tci.setLeft((int)copyLeft.getValue());
            tci.setMemorySlot((int)copySlot.getValue());
            operations.add(tci);
        }
        if (cbStoreImage.isSelected()) {
            TdpStoreImage tsi = new TdpStoreImage();
            tsi.setMemorySlot((int)storeSlot.getValue());
            operations.add(tsi);
        }

        return new TdpManipulateMemorySlot(operations);
    }
}
