/*
Copyright (c) 2013-2019, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.simulator.ui.panels;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.disperanto.data.TdpSetText;
import org.disperanto.data.TdpSetText.TdpTextLine;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpTextAlignment;
import org.disperanto.simulator.ui.models.ISetTextModel;

import net.miginfocom.swing.MigLayout;

/**
 * Panel showing text lines.
 */
public class SetTextPanel extends JPanel implements ISetTextModel {

    private final JTextField line1Field;
    private final JComboBox<ETdpTextAlignment> line1Alignment;
    private final JTextField line2Field;
    private final JComboBox<ETdpTextAlignment> line2Alignment;
    private final JTextField line3Field;
    private final JComboBox<ETdpTextAlignment> line3Alignment;

    public SetTextPanel() {
        super(new MigLayout("", "[grow, align right][110][]", "[][][]"));
        setBorder(BorderFactory.createTitledBorder("Set text"));
        add(new JLabel("Line 1: "));
        line1Field = new JTextField();
        add(line1Field, "grow");
        line1Alignment = new JComboBox<>(ETdpTextAlignment.values());
        add(line1Alignment, "grow, wrap");

        add(new JLabel("Line 2: "));
        line2Field = new JTextField();
        add(line2Field, "grow");
        line2Alignment = new JComboBox<>(ETdpTextAlignment.values());
        add(line2Alignment, "grow, wrap");

        add(new JLabel("Line 3: "));
        line3Field = new JTextField();
        add(line3Field, "grow");
        line3Alignment = new JComboBox<>(ETdpTextAlignment.values());
        add(line3Alignment, "grow, wrap");
    }

    @Override
    public TdpSetText getSetText() {
        TdpSetText setText = new TdpSetText();
        List<TdpTextLine> lines = new ArrayList<>();
        if (!line1Field.getText().equals("")) {
            lines.add(new TdpTextLine((ETdpTextAlignment)line1Alignment.getSelectedItem(), line1Field.getText()));
        }
        if (!line2Field.getText().equals("")) {
            lines.add(new TdpTextLine((ETdpTextAlignment)line2Alignment.getSelectedItem(), line2Field.getText()));
        }
        if (!line3Field.getText().equals("")) {
            lines.add(new TdpTextLine((ETdpTextAlignment)line3Alignment.getSelectedItem(), line3Field.getText()));
        }
        setText.setLines(lines);
        return setText;
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.SET_TEXT));
    }

    private void setComponentsEnabled(boolean enabled) {
        line1Field.setEnabled(enabled);
        line1Alignment.setEnabled(enabled);
        line2Field.setEnabled(enabled);
        line2Alignment.setEnabled(enabled);
        line3Field.setEnabled(enabled);
        line3Alignment.setEnabled(enabled);
    }
}
