/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this
   list of conditions and the following disclaimer in the documentation and/or
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 */
package org.disperanto.simulator.ui.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedHashMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import net.miginfocom.swing.MigLayout;

import org.disperanto.data.TdpStartSlideShow;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpSlideShowMode;
import org.disperanto.simulator.ui.components.NumericTextField;
import org.disperanto.simulator.ui.models.ISlideShowModel;

/**
 * Panel to select the images of a slide show.
 */
public class SlideShowPanel extends JPanel implements ISlideShowModel {

    private JComboBox<ETdpSlideShowMode> modeBox;
    private JTable slotTimeTable;
    private NumericTextField slotField;
    private NumericTextField timeField;
    private JButton addButton;
    private JButton clearButton;

    public SlideShowPanel() {
        super(new MigLayout("", "[align right][][align right][][align right]", ""));
        initGui();
    }

    @Override
    public TdpStartSlideShow getSlideShowInfo() {
        TdpStartSlideShow slideShowInfo = new TdpStartSlideShow();
        slideShowInfo.setMode((ETdpSlideShowMode)modeBox.getSelectedItem());
        slideShowInfo.setImageSlotsSetting(validateSlotSettings());
        return slideShowInfo;
    }

    private LinkedHashMap<Integer, Integer> validateSlotSettings() {
        if (slotTimeTable.getRowCount() < 1) {
            JOptionPane.showMessageDialog(this, "No slideshow settings set.", "Empty slideshow settings",
                    JOptionPane.ERROR_MESSAGE);
        }

        LinkedHashMap<Integer, Integer> slotSettings = new LinkedHashMap<Integer, Integer>();

        for (int i = 0; i < slotTimeTable.getModel().getRowCount(); i++) {
            for (int j = 0; j < slotTimeTable.getModel().getColumnCount(); j++) {
                try {
                    Integer.parseInt(slotTimeTable.getValueAt(i, j).toString());
                } catch (NumberFormatException e) {
                    JOptionPane.showMessageDialog(this, "Wrong format in time settings, only numbers only",
                            "Wrong format", JOptionPane.ERROR_MESSAGE);
                    slotTimeTable.setRowSelectionInterval(i, i);
                }
            }
            slotSettings.put(Integer.parseInt(slotTimeTable.getValueAt(i, 0).toString()),
                    Integer.parseInt(slotTimeTable.getValueAt(i, 1).toString()));
        }

        return slotSettings;
    }

    @Override
    public void commandListChanged(Collection<ETdpCommand> commands) {
        setComponentsEnabled(!commands.contains(ETdpCommand.START_SLIDE_SHOW));
    }

    private void setComponentsEnabled(boolean enabled) {
        modeBox.setEnabled(enabled);
        slotTimeTable.setEnabled(enabled);
        slotField.setEnabled(enabled);
        timeField.setEnabled(enabled);
        addButton.setEnabled(enabled);
        clearButton.setEnabled(enabled);
    }

    private void initGui() {
        setBorder(BorderFactory.createTitledBorder("Slide show settings"));
        modeBox = new JComboBox<ETdpSlideShowMode>(ETdpSlideShowMode.values());
        add(modeBox, "span 5, wrap");

        String[] columnNames = { "Slot", "Time (tenth of second)" };
        Object[][] initData = new Object[][] { { 2, 50 }, { 1, 10 } };
        slotTimeTable = new JTable(new DefaultTableModel(initData, columnNames));
        add(new JScrollPane(slotTimeTable), "span 5, wrap, grow");

        add(new JLabel("Slot:"));
        slotField = new NumericTextField(0, 999);
        add(slotField);

        add(new JLabel("Time:"));
        timeField = new NumericTextField(0, Integer.MAX_VALUE);
        add(timeField);

        addButton = new JButton("Add");
        addButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((DefaultTableModel)slotTimeTable.getModel()).addRow(new Object[] { slotField.getValue(),
                        timeField.getValue() });
            }
        });
        add(addButton, "wrap");

        clearButton = new JButton("Clear slot settings");
        clearButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                ((DefaultTableModel)slotTimeTable.getModel()).setRowCount(0);
            }
        });
        add(clearButton, "skip 4");
    }
}
