/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.communication;

import java.io.IOException;

import org.disperanto.enums.EIpConnectionType;

/**
 * Extension of {@link IConnection} that can setup and shutdown a connection itself.
 */
public interface IStartableConnection extends IConnection {

    /**
     * Opens the connection for usage, after calling this method the {@link #send(java.nio.ByteBuffer)} and
     * {@link #getInputStream()} methods can be called.
     */
    void start() throws IOException;

    /**
     * Closes the connection for usage, after calling this method the {@link #send(java.nio.ByteBuffer)} and
     * {@link #getInputStream()} methods cannot be called anymore.
     */
    void stop();
    
    /**
     * Returns the IP address from which a message is received.
     *
     * @return the connection type used
     */
    EIpConnectionType getConnectionType();

    /**
     * Returns the IP address from which a message is received.
     *
     * @return IP address
     */
    String getIpAddress();

    /**
     * Returns the IP address from which a message is received.
     *
     * @return the port used for the connection
     */
    int getPort();

}
