/*
Copyright (c) 2013-2019, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import java.nio.ByteBuffer;

import org.disperanto.enums.ETdpDisplayType;
import org.disperanto.utils.ByteBufferUtils;


/**
 * Defines the display properties in Disperanto.
 */
public class TdpDisplayProperties {

    private int displayAddress;
    private int protocolVersion;
    private ETdpDisplayType displayType;
    private String supplierAndProductType;
    private String serialNumber;
    private String softwareVersion;
    private boolean externalLighting = false;
    private int displayHeightInPixels = 0;
    private int displayWidthInPixels = 0;
    private int nrFixedImages = 0;
    private int nrWritableImages = 0;
    private boolean slideShow = false;
    private int nrSlideShowImages = 0;
    private ByteBuffer rgbColors = null;
    private ByteBuffer fixedColorPalette = null;
    private boolean pngSupported = false;
    private int textDisplayRows = 0;
    private int textDisplayColumns = 0;

    /**
     * Constructor
     */
    public TdpDisplayProperties() {
        // empty object
    }

    /**
     * Copy constructor.
     *
     * @param copyObject Object to copy
     */
    public TdpDisplayProperties(TdpDisplayProperties copyObject) {
        this.displayAddress = copyObject.displayAddress;
        this.protocolVersion = copyObject.protocolVersion;
        this.displayType = copyObject.displayType;
        this.supplierAndProductType = copyObject.supplierAndProductType;
        this.serialNumber = copyObject.serialNumber;
        this.softwareVersion = copyObject.softwareVersion;
        this.externalLighting = copyObject.externalLighting;
        this.displayHeightInPixels = copyObject.displayHeightInPixels;
        this.displayWidthInPixels = copyObject.displayWidthInPixels;
        this.nrFixedImages = copyObject.nrFixedImages;
        this.nrWritableImages = copyObject.nrWritableImages;
        this.slideShow = copyObject.slideShow;
        this.rgbColors = copyObject.rgbColors;
        this.fixedColorPalette = copyObject.fixedColorPalette;
        this.pngSupported = copyObject.pngSupported;
        this.textDisplayRows = copyObject.textDisplayRows;
        this.textDisplayColumns = copyObject.textDisplayColumns;
    }

    /**
     * @return the protocolVersion
     */
    public final int getDisplayAddress() {
        return displayAddress;
    }

    /**
     * @param displayAddress Address of the display
     */
    public final void setDisplayAddress(int displayAddress) {
        this.displayAddress = displayAddress;
    }

    /**
     * @return the protocolVersion
     */
    public final int getProtocolVersion() {
        return protocolVersion;
    }

    /**
     * @param protocolVersion the protocolVersion to set
     */
    public final void setProtocolVersion(int protocolVersion) {
        this.protocolVersion = protocolVersion;
    }

    /**
     * @return the displayType
     */
    public final ETdpDisplayType getDisplayType() {
        return displayType;
    }

    /**
     * @param displayType the displayType to set
     */
    public final void setDisplayType(ETdpDisplayType displayType) {
        this.displayType = displayType;
    }

    /**
     * @return the supplierAndProductType
     */
    public final String getSupplierAndProductType() {
        return supplierAndProductType;
    }

    /**
     * @param supplierAndProductType the supplierAndProductType to set
     */
    public final void setSupplierAndProductType(String supplierAndProductType) {
        this.supplierAndProductType = supplierAndProductType;
    }

    /**
     * @return the serialNumber
     */
    public final String getSerialNumber() {
        return serialNumber;
    }

    /**
     * @param serialNumber the serialNumber to set
     */
    public final void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    /**
     * @return the softwareVersion
     */
    public final String getSoftwareVersion() {
        return softwareVersion;
    }

    /**
     * @param softwareVersion the softwareVersion to set
     */
    public final void setSoftwareVersion(String softwareVersion) {
        this.softwareVersion = softwareVersion;
    }

    /**
     * @return the externalLighting
     */
    public final boolean hasExternalLighting() {
        return externalLighting;
    }

    /**
     * @param externalLighting the externalLighting to set
     */
    public final void setExternalLighting(boolean externalLighting) {
        this.externalLighting = externalLighting;
    }

    /**
     * @return the displayHeightInPixels
     */
    public final int getDisplayHeightInPixels() {
        return displayHeightInPixels;
    }

    /**
     * @param displayHeightInPixels the displayHeightInPixels to set
     */
    public final void setDisplayHeightInPixels(int displayHeightInPixels) {
        this.displayHeightInPixels = displayHeightInPixels;
    }

    /**
     * @return the displayWidthInPixels
     */
    public final int getDisplayWidthInPixels() {
        return displayWidthInPixels;
    }

    /**
     * @param displayWidthInPixels the displayWidthInPixels to set
     */
    public final void setDisplayWidthInPixels(int displayWidthInPixels) {
        this.displayWidthInPixels = displayWidthInPixels;
    }

    /**
     * @return the nrFixedImages
     */
    public final int getNrFixedImages() {
        return nrFixedImages;
    }

    /**
     * @param nrFixedImages the nrFixedImages to set
     */
    public final void setNrFixedImages(int nrFixedImages) {
        this.nrFixedImages = nrFixedImages;
    }

    /**
     * @return the nrWritableImages
     */
    public final int getNrWritableImages() {
        return nrWritableImages;
    }

    /**
     * @param nrWritableImages the nrWritableImages to set
     */
    public final void setNrWritableImages(int nrWritableImages) {
        this.nrWritableImages = nrWritableImages;
    }

    public boolean isWritableImage(int imageSlot) {
        // Writable images are stored after fixed images
        int writableIndex = imageSlot - nrFixedImages;
        return writableIndex >= 0 && writableIndex < nrWritableImages;
    }

    public boolean isFixedImage(int imageSlot) {
        return imageSlot >= 0 && imageSlot < nrFixedImages;
    }

    /**
     * @return the slideShow
     */
    public final boolean hasSlideShow() {
        return slideShow;
    }

    /**
     * @param slideShow the slideShow to set
     */
    public final void setSlideShow(boolean slideShow) {
        this.slideShow = slideShow;
    }

    /**
     * @return the nrSlideShowImages
     */
    public final int getNrSlideShowImages() {
        return nrSlideShowImages;
    }

    /**
     * @param nrSlideShowImages the nrSlideShowImages to set
     */
    public final void setNrSlideShowImages(int nrSlideShowImages) {
        this.nrSlideShowImages = nrSlideShowImages;
    }

    /**
     * @return the rgbColors
     */
    public final ByteBuffer getRgbColors() {
        return rgbColors;
    }

    /**
     * @param rgbColors the rgbColors to set
     */
    public final void setRgbColors(ByteBuffer rgbColors) {
        this.rgbColors = rgbColors;
    }

    /**
     * @return the fixedColorPalette
     */
    public final ByteBuffer getFixedColorPalette() {
        return fixedColorPalette;
    }

    /**
     * @param fixedColorPalette the fixedColorPalette to set
     */
    public final void setFixedColorPalette(ByteBuffer fixedColorPalette) {
        this.fixedColorPalette = fixedColorPalette;
    }

    /**
     * @return the pngSupported
     */
    public final boolean isPngSupported() {
        return pngSupported;
    }

    /**
     * @param pngSupported the pngSupported to set
     */
    public final void setPngSupported(boolean pngSupported) {
        this.pngSupported = pngSupported;
    }

    /**
     * @return the textDisplayRows
     */
    public int getTextDisplayRows() {
        return textDisplayRows;
    }

    /**
     * @param textDisplayRows the textDisplayRows to set
     */
    public void setTextDisplayRows(int textDisplayRows) {
        this.textDisplayRows = textDisplayRows;
    }

    /**
     * @return the textDisplayColumns
     */
    public int getTextDisplayColumns() {
        return textDisplayColumns;
    }

    /**
     * @param textDisplayColumns the textDisplayColumns to set
     */
    public void setTextDisplayColumns(int textDisplayColumns) {
        this.textDisplayColumns = textDisplayColumns;
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder("DisplayProperties :");
        builder.append("\n\tProtocol version: ").append(protocolVersion);
        builder.append("\n\tDisplay type: ").append(displayType.getName());
        builder.append("\n\tSupplier and product type: ").append(supplierAndProductType);
        builder.append("\n\tSerial number: ").append(serialNumber);
        builder.append("\n\tSoftware version: ").append(softwareVersion);
        builder.append("\n\tExternal Lighting: ").append(externalLighting);
        if (displayType == ETdpDisplayType.MATRIX) {
            builder.append("\n\tDisplay height: ").append(displayHeightInPixels);
            builder.append("\n\tdisplay width: ").append(displayWidthInPixels);
        }
        builder.append("\n\tNumber of fixed images: ").append(nrFixedImages);
        builder.append("\n\tNumber of writable images: ").append(nrWritableImages);
        builder.append("\n\tSlide show: ").append(slideShow);
        if (slideShow) {
            builder.append("\n\t\tNr slideshow images : ").append(nrSlideShowImages);
        }
        if (rgbColors != null) {
            builder.append("\n\tRGB Colors : ").append(ByteBufferUtils.toByteString(rgbColors));
        } else {
            builder.append("\n\tFixed color palette : ").append(ByteBufferUtils.toByteString(fixedColorPalette));
        }
        builder.append("\n\tPNG supported : ").append(pngSupported);
        if (displayType == ETdpDisplayType.TEXT) {
            builder.append("\n\tDisplay number of lines: ").append(textDisplayRows);
            builder.append("\n\tdisplay number of characters: ").append(textDisplayColumns);
        }

        return builder.toString();
    }

}
