/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import java.util.Arrays;

import org.disperanto.enums.ETdpImageType;
import org.disperanto.enums.ETdpManipulateMemorySlot;

/**
 * Defines the location and image for Load Image in Disperanto.
 */
public class TdpLoadImage implements IImageOperation {

    private int left;
    private int top;
    private ETdpImageType imageType;
    private byte[] image;

    /**
     * Constructor
     */
    public TdpLoadImage() {
        // empty object
    }
    
    public TdpLoadImage(ETdpImageType imageType, byte[] image) {
        this.imageType = imageType;
        this.image = (image != null ? image.clone() : null);
    }

    @Override
    public ETdpManipulateMemorySlot getType() {
        return ETdpManipulateMemorySlot.LOAD_IMAGE;
    }

    /**
     * @return the left
     */
    public final int getLeft() {
        return left;
    }

    /**
     * @param left the left to set
     */
    public final void setLeft(int left) {
        this.left = left;
    }

    /**
     * @return the top
     */
    public final int getTop() {
        return top;
    }

    /**
     * @param top the top to set
     */
    public final void setTop(int top) {
        this.top = top;
    }

    /**
     * @return the imageType
     */
    public final ETdpImageType getImageType() {
        return imageType;
    }

    /**
     * @param imageType the imageType to set
     */
    public final void setImageType(ETdpImageType imageType) {
        this.imageType = imageType;
    }

    /**
     * @return the image
     */
    public final byte[] getImage() {
        return (image != null ? image.clone() : null);
    }

    /**
     * @param image the image to set
     */
    public final void setImage(byte[] image) {
        this.image = (image != null ? image.clone() : null);
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder("Load image: ");

        builder.append("\n\tleft: ").append(left);
        builder.append("\n\ttop: ").append(top);
        builder.append("\n\ttype: ").append(imageType);
        builder.append("\n\timage: ").append(Arrays.toString(image));

        return builder.toString();
    }
}
