/*
Copyright (c) 2013-2019, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.disperanto.enums.ETdpTextAlignment;

/**
 * Defines the set text operation in Disperanto.
 */
public class TdpSetText {
    private List<TdpTextLine> lines = Collections.emptyList();

    /**
     * Constructor
     */
    public TdpSetText() {
        // empty object
    }

    public TdpSetText(List<TdpTextLine> lines) {
        this.lines = new ArrayList<>(lines);
    }

    /**
     * @return the text lines
     */
    public final List<TdpTextLine> getLines() {
        return Collections.unmodifiableList(lines);
    }

    /**
     * @param lines the lines to set
     */
    public final void setLines(List<TdpTextLine> lines) {
        this.lines = new ArrayList<>(lines);
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder("Set text: ");

        for (TdpTextLine line : lines) {
            builder.append("\n\t").append(line.toString());
        }

        return builder.toString();
    }

    public static final class TdpTextLine {
        private ETdpTextAlignment alignment;
        private String text;

        public TdpTextLine() {
            // set defaults
            alignment = ETdpTextAlignment.LEFT;
            text = "";
        }

        public TdpTextLine(ETdpTextAlignment alignment, String text) {
            this.alignment = alignment;
            this.text = text;
        }

        public ETdpTextAlignment getAlignment() {
            return alignment;
        }

        public void setAlignment(ETdpTextAlignment alignment) {
            this.alignment = alignment;
        }

        public String getText() {
            return text;
        }

        public void setText(String text) {
            this.text = text;
        }

        @Override
        public final String toString() {
            final StringBuilder builder = new StringBuilder("Text line: ");

            builder.append("\n\talignment: ").append(alignment);
            builder.append("\n\ttext: ").append(text);

            return builder.toString();
        }
    }
}
