/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import java.util.LinkedHashMap;
import java.util.Map.Entry;

import org.disperanto.enums.ETdpSlideShowMode;


/**
 * Defines the settings for a StartSlideShow command in Disperanto.
 */
public class TdpStartSlideShow {

    private ETdpSlideShowMode mode;
    // map of slots and duration. LinkedHashMap to preserve order.
    private LinkedHashMap<Integer, Integer> imageSlotsSettings = new LinkedHashMap<Integer, Integer>();

    /**
     * Constructor
     */
    public TdpStartSlideShow() {
        // empty object
    }

    /**
     * @return the mode
     */
    public final ETdpSlideShowMode getMode() {
        return mode;
    }

    /**
     * @param mode the mode to set
     */
    public final void setMode(ETdpSlideShowMode mode) {
        this.mode = mode;
    }

    /**
     * @return the imageSlotsSetting
     */
    public final LinkedHashMap<Integer, Integer> getImageSlotsSetting() {
        return imageSlotsSettings;
    }

    /**
     * @param imageSlotsSettings the imageSlotsSetting to set
     */
    public final void setImageSlotsSetting(LinkedHashMap<Integer, Integer> imageSlotsSettings) {
        this.imageSlotsSettings = imageSlotsSettings;
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder();
        builder.append("mode: ").append(mode);
        builder.append("\n\timagelist: ");

        for (Entry<Integer, Integer> slide : imageSlotsSettings.entrySet()) {
            builder.append("\n\t   slot: ").append(slide.getKey())
                .append(", time in tenth of a second: ").append(slide.getValue());
        }

        return builder.toString();
    }

}
