/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Defines the display status in Disperanto.
 */
public class TdpStatus {

    private Map<Integer, Short> shownImages = new HashMap<Integer, Short>();
    private Integer brightnessInPercentage = null;
    private Integer intensityExternalLighting = null;
    private List<Integer> measuredLightIntensity = new ArrayList<Integer>();
    private String gpsPosition = "";
    private Integer temperature = null;
    private Integer heating = null;
    private Integer cooling = null;

    /**
     * Constructor
     */
    public TdpStatus() {
        // empty object
    }

    /**
     * Copy constructor.
     *
     * @param copyObject Object to copy
     */
    public TdpStatus(TdpStatus copyObject) {
        this.shownImages = copyObject.shownImages;
        this.brightnessInPercentage = copyObject.brightnessInPercentage;
        this.intensityExternalLighting = copyObject.intensityExternalLighting;
        this.measuredLightIntensity = copyObject.measuredLightIntensity;
        this.gpsPosition = copyObject.gpsPosition;
        this.temperature = copyObject.temperature;
        this.heating = copyObject.heating;
        this.cooling = copyObject.cooling;
    }

    /**
     * @return the shownImage
     */
    public final Map<Integer, Short> getShownImages() {
        return shownImages;
    }

    /**
     * @param shownImages the shownImage to set
     */
    public final void setShownImages(Map<Integer, Short> shownImages) {
        this.shownImages = shownImages;
    }

    /**
     * @return the brightnessInPercentage
     */
    public final Integer getBrightnessInPercentage() {
        return brightnessInPercentage;
    }

    /**
     * @param brightnessInPercentage the brightnessInPercentage to set
     */
    public final void setBrightnessInPercentage(Integer brightnessInPercentage) {
        this.brightnessInPercentage = brightnessInPercentage;
    }

    /**
     * @return the intensityExternalLighting
     */
    public final Integer getIntensityExternalLighting() {
        return intensityExternalLighting;
    }

    /**
     * @param intensityExternalLighting the intensityExternalLighting to set
     */
    public final void setIntensityExternalLighting(Integer intensityExternalLighting) {
        this.intensityExternalLighting = intensityExternalLighting;
    }

    /**
     * @return the measuredLightIntensity
     */
    public final List<Integer> getMeasuredLightIntensity() {
        return measuredLightIntensity;
    }

    /**
     * @param measuredLightIntensity the measuredLightIntensity to set
     */
    public final void setMeasuredLightIntensity(List<Integer> measuredLightIntensity) {
        this.measuredLightIntensity = measuredLightIntensity;
    }

    /**
     * @return the gpsPosition
     */
    public final String getGpsPosition() {
        return gpsPosition;
    }

    /**
     * @param gpsPosition the gpsPosition to set
     */
    public final void setGpsPosition(String gpsPosition) {
        this.gpsPosition = gpsPosition;
    }

    /**
     * @return the temperature
     */
    public final Integer getTemperature() {
        return temperature;
    }

    /**
     * @param temperature the temperature to set
     */
    public final void setTemperature(Integer temperature) {
        this.temperature = temperature;
    }

    /**
     * @return the heating
     */
    public final Integer getHeating() {
        return heating;
    }

    /**
     * @param heating the heating to set
     */
    public final void setHeating(Integer heating) {
        this.heating = heating;
    }

    /**
     * @return the cooling
     */
    public final Integer getCooling() {
        return cooling;
    }

    /**
     * @param cooling the cooling to set
     */
    public final void setCooling(Integer cooling) {
        this.cooling = cooling;
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder("Status :");
        if (shownImages.size() == 0) {
            builder.append("\n\tShown image(s): none");
        } else {
            builder.append("\n\tShown image(s): ");
            for (Entry<Integer, Short> entry : shownImages.entrySet()) {
                final Integer slot = entry.getKey();
                final Short crc = entry.getValue();
                builder.append("\n\t\tSlot").append(slot).append(": CRC = ").append(crc);
            }
        }
        builder.append("\n\tBrightness: ").append(brightnessInPercentage);
        builder.append("\n\tIntensity external lighting: ").append(intensityExternalLighting);
        if (measuredLightIntensity.size() > 0) {
            builder.append("\n\tMeasured light intensity: ");
            for (int i = 0; i < measuredLightIntensity.size(); i++) {
                builder.append("\n\t\tSensor ").append(i).append(": ")
                    .append(measuredLightIntensity.get(i)).append("%");
            }
        } else {
            builder.append("\n\tMeasured light intensity: no sensors available");
        }
        builder.append("\n\tGPS position: ").append(gpsPosition);
        builder.append("\n\tTemperature: ").append(temperature);
        if (heating != null) {
            builder.append("\n\tHeating: ").append(heating.toString());
        } else {
            builder.append("\n\tHeating: not available");
        }
        if (cooling != null) {
            builder.append("\n\tCooling: ").append(cooling.toString());
        } else {
            builder.append("\n\tCooling: not available");
        }

        return builder.toString();
    }
}
