/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.data;

import org.disperanto.enums.ETdpTimeoutMode;

/**
 * Defines the settings for a communication timeout in Disperanto.
 */
public class TdpTimeoutSettings {

    private ETdpTimeoutMode timeoutMode = ETdpTimeoutMode.NO_TIMEOUT;
    private int timeoutInSec = -1;
    private int imageSlot = -1;

    /**
     * Constructor
     */
    public TdpTimeoutSettings() {
        // empty object
    }

    /**
     * @return the timeoutMode
     */
    public final ETdpTimeoutMode getTimeoutMode() {
        return timeoutMode;
    }

    /**
     * @param timeoutMode the timeoutMode to set
     */
    public final void setTimeoutMode(ETdpTimeoutMode timeoutMode) {
        this.timeoutMode = timeoutMode;
    }

    /**
     * @return the timeoutInSec
     */
    public final int getTimeoutInSec() {
        return timeoutInSec;
    }

    /**
     * @param timeoutInSec the timeoutInSec to set
     */
    public final void setTimeoutInSec(int timeoutInSec) {
        this.timeoutInSec = timeoutInSec;
    }

    /**
     * @return the imageSlot
     */
    public final int getImageSlot() {
        return imageSlot;
    }

    /**
     * @param imageSlot the imageSlot to set
     */
    public final void setImageSlot(int imageSlot) {
        this.imageSlot = imageSlot;
    }

    @Override
    public final String toString() {
        final StringBuilder builder = new StringBuilder();
        builder.append("Timeout mode: ").append(timeoutMode.getDescription());

        if (timeoutMode != ETdpTimeoutMode.NO_TIMEOUT) {
            builder.append("\n\tTimeout in seconds: ").append(timeoutInSec);
        }

        if (timeoutMode == ETdpTimeoutMode.SHOW_IMAGE) {
            builder.append("\n\tImage slot: ").append(imageSlot);
        }

        return builder.toString();
    }
}
