/*
Copyright (c) 2013-2019, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.enums;

import org.omg.CORBA.UNKNOWN;


/**
 * The commands in Disperanto.
 */
public enum ETdpCommand {

    NOTIFICATION(0x00, "Notification"),
    DISPLAY_PROPERTIES(0x01, "DisplayProperties"),
    STATUS(0x02, "Status"),
    REBOOT(0x03, "Reboot"),
    KEEP_ALIVE(0x04, "KeepAlive"),
    SET_COMMUNICATION_TIMEOUT(0x05, "SetCommunicationTimeout"),
    SET_BRIGHTNESS(0x06, "SetBrightness"),
    SET_EXTERNAL_LIGHTING(0x07, "SetExternalLighting"),
    DIAGNOSTICS(0x08, "Diagnostics"),
    MANIPULATE_MEMORY_SLOT(0x10, "ManipulateMemorySlot"),
    CALCULATE_CRC_IMAGES(0x11, "CalculateCrcImages"),
    SHOW_NO_IMAGE(0x12, "ShowNoImage"),
    SHOW_IMAGE(0x13, "ShowImage"),
    START_SLIDE_SHOW(0x14, "ShowSlideShow"),
    SET_TEXT(0x20, "SetText"),
    SUPPLIER_SERVICE_MODE(0x30, "SupplierServiceMode"),
    UNKNOWN(0xFF, "Unknown");

    /**
     * Numerical value of command in protocol.
     */
    private final int value;

    /**
     * Name of command in protocol.
     */
    private final String name;

    /**
     * Private constructor
     * 
     * @param value Numerical value of command in protocol
     * @param name Name of command in protocol
     */
    private ETdpCommand(int value, String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * @return Numerical value of command in protocol
     */
    public int getValue() {
        return value;
    }

    /**
     * @return Name of the command.
     */
    public String getName() {
        return name;
    }

    /**
     * Converts a value to a Disperanto Command enum.
     * 
     * @param value Numerical value of command in protocol
     * @return command enum with given value, or {@link UNKNOWN} if the given value matches no known command type.
     */
    public static ETdpCommand convertValue(int value) {
        for (ETdpCommand current : values()) {
            if (current.getValue() == value) {
                return current;
            }
        }
        return ETdpCommand.UNKNOWN;
    }

}
