/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.enums;

/**
 * Different message types in Disperanto.
 */
public enum ETdpMessageType {

    COMMAND(0x01, "Command"),
    RESPONSE(0x00, "response");

    /**
     * Numerical value of message type in protocol
     */
    private final int value;

    /**
     * Name of message type in protocol
     */
    private final String name;

    /**
     * Private constructor
     *
     * @param value Numerical value of message type in protocol
     * @param message Name of message type in protocol
     */
    private ETdpMessageType(int value, String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * @return Name of message type in protocol
     */
    public String getName() {
        return name;
    }

    /**
     * @return Numerical value of message type in protocol
     */
    public int getValue() {
        return value;
    }

    /**
     * Converts a value to a ETdpMessageType enum.
     *
     * @param value Numerical value of message type in protocol
     * @return Message type enum with given value, or null if no enum constant was found with the given value.
     */
    public static ETdpMessageType convertValue(int value) {
        for (ETdpMessageType current : values()) {
            if (current.getValue() == value) {
                return current;
            }
        }

        return null;
    }
}
