/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.enums;

/**
 * Definition of timeout modes in Disperanto.
 */
public enum ETdpTimeoutMode {

    NO_TIMEOUT(0x00, "No timeout"),
    CLEAR_DISPLAY(0x01, "Clear display after timeout"),
    SHOW_IMAGE(0x02, "Show image after timeout");

    /**
     * Numerical value of timeout mode
     */
    private final int value;

    /**
     * Description of timeout mode in protocol
     */
    private final String description;

    /**
     * Private constructor
     *
     * @param value Numerical value of timeout mode in protocol
     * @param description Description of timeout mode in protocol
     */
    private ETdpTimeoutMode(int value, String description) {
        this.value = value;
        this.description = description;
    }

    /**
     * @return Numerical value of timeout mode in protocol
     */
    public int getValue() {
        return value;
    }

    /**
     * @return Description of timeout mode in protocol
     */
    public String getDescription() {
        return description;
    }

    /**
     * Converts a value to a timeout mode enum.
     *
     * @param value Numerical value of timeout mode in protocol
     * @return enum with given value, or null if no enum constant was found with the given value.
     */
    public static ETdpTimeoutMode convertValue(int value) {
        for (ETdpTimeoutMode current : values()) {
            if (current.getValue() == value) {
                return current;
            }
        }

        return null;
    }
}
