/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.messages;

import java.nio.ByteBuffer;

import org.disperanto.enums.ETdpCommand;
import org.disperanto.exception.TdpProtocolException;


/**
 * Formatter for the ManipulateMemorySlot response.
 */
public final class ManipulateMemorySlotResponse {
    /**
     * Constructor
     */
    private ManipulateMemorySlotResponse() {
        // Empty constructor
    }

    /**
     * Creates the data buffer for the ManipulateMemorySlot response.
     * 
     * @param crc CRC of image in working memory
     * @return Data buffer for the CRC
     */
    public static ByteBuffer createData(short crc) {
        final ByteBuffer data = ByteBuffer.allocate(2);

        data.putShort(crc);

        return data;
    }

    /**
     * Parses the received data to a ManipulateMemorySlotResponse
     *
     * @param message Generic message that is received from a display
     * @return CRC of the image in the working memory
     */
    public static short parse(TdpMessage message) throws TdpProtocolException {
        final TdpFormatter formatter = new TdpFormatter();
        short crc = 0;

        formatter.checkResponse(message, ETdpCommand.MANIPULATE_MEMORY_SLOT);

        // Parse data
        final ByteBuffer data = formatter.getMessageData(message);

        crc = data.getShort();

        formatter.checkRemainingData(data);

        return crc;
    }
}
