/*
Copyright (c) 2013-2015, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.messages;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import org.disperanto.enums.ETdpCommand;
import org.disperanto.exception.TdpProtocolException;

public final class StartSlideShowResponse {

    /**
     * Constructor
     */
    private StartSlideShowResponse() {
        // Empty constructor
    }

    /**
     * Creates the data buffer for the StartSlideShow response.
     * 
     * @param crcs List of checksums of the image slots that were requested in the slideshow
     * @return Data buffer for the crc's
     */
    public static ByteBuffer createData(List<Short> crcs) {
        final ByteBuffer data = ByteBuffer.allocate(2 * crcs.size());

        for (Short crc : crcs) {
            data.putShort(crc.shortValue());
        }

        return data;
    }

    /**
     * Parses the received data to a array list of checksums of the image slots that were requested in the slideshow
     *
     * @param message Generic message that is received from a display
     * @return List of checksums of the image slots that were requested
     */
    public static List<Short> parse(TdpMessage message) throws TdpProtocolException {
        final TdpFormatter formatter = new TdpFormatter();
        final List<Short> crcList = new ArrayList<Short>();

        formatter.checkResponse(message, ETdpCommand.START_SLIDE_SHOW);

        // Parse data
        final ByteBuffer data = formatter.getMessageData(message);

        while (data.hasRemaining()) {
            crcList.add(data.getShort());
        }

        formatter.checkRemainingData(data);

        return crcList;
    }

}
