package org.disperanto.communication;

import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.nio.ByteBuffer;

import org.disperanto.enums.EIpConnectionType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LoopbackConnection implements IStartableConnection {

    private static final Logger LOG = LoggerFactory.getLogger(LoopbackConnection.class);

    private PipedInputStream in;
    private PipedOutputStream out;

    @Override
    public void send(ByteBuffer data) throws IOException {
        out.write(data.array());
    }

    @Override
    public InputStream getInputStream() {
        return in;
    }

    @Override
    public void start() throws IOException {
        out = new PipedOutputStream();
        in = new PipedInputStream(out);
    }

    @Override
    public void stop() {
        if (in != null) {
            try {
                in.close();
            } catch (IOException e) {
                LOG.warn("Error closing input stream");
            }
        }
        if (out != null) {
            try {
                out.close();
            } catch (IOException e) {
                LOG.warn("Error closing output stream");
            }
        }
    }

    @Override
    public EIpConnectionType getConnectionType() {
        return EIpConnectionType.UDP;
    }

    @Override
    public String getIpAddress() {
        return "localhost";
    }

    @Override
    public int getPort() {
        return 3456;
    }

}
