/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.messages;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import org.disperanto.data.IImageOperation;
import org.disperanto.data.TdpClearRectangle;
import org.disperanto.data.TdpCopyImage;
import org.disperanto.data.TdpInitializeWorkingMemory;
import org.disperanto.data.TdpLoadImage;
import org.disperanto.data.TdpManipulateMemorySlot;
import org.disperanto.data.TdpStoreImage;
import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpImageType;
import org.disperanto.enums.ETdpMessageType;
import org.disperanto.exception.TdpProtocolException;
import org.disperanto.utils.TdpUtils;
import org.disperanto.utils.VlqUtils;
import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * JUnit tests for class TdpCommandManipulateMemorySlot.
 */
public final class TdpCommandManipulateMemorySlotTest {

    @Rule
    public ExpectedException thrown = ExpectedException.none();

    /**
     * Tests the function parseData()
     */
    @Test
    public void testParseData() throws TdpProtocolException {
        ValidMessage validMessage = new ValidMessage();
        TdpMessage tdpMessage = validMessage.buildTdpMessage();

        TdpManipulateMemorySlot actions = ManipulateMemorySlotCommand.parse(tdpMessage);

        int step = 0;
        TdpInitializeWorkingMemory init0 = (TdpInitializeWorkingMemory)actions.getOperations().get(step++);
        TdpInitializeWorkingMemory initializeWorkingMemory = validMessage.initializeWorkingMemory;
        Assert.assertEquals(initializeWorkingMemory.getWidth(), init0.getWidth());
        Assert.assertEquals(initializeWorkingMemory.getHeight(), init0.getHeight());

        TdpClearRectangle clearOp0 = (TdpClearRectangle)actions.getOperations().get(step++);
        TdpClearRectangle clearRectangle = validMessage.clearRectangle;
        Assert.assertEquals(clearRectangle.getLeft(), clearOp0.getLeft());
        Assert.assertEquals(clearRectangle.getTop(), clearOp0.getTop());
        Assert.assertEquals(clearRectangle.getWidth(), clearOp0.getWidth());
        Assert.assertEquals(clearRectangle.getHeight(), clearOp0.getHeight());

        TdpClearRectangle clearOp1 = (TdpClearRectangle)actions.getOperations().get(step++);
        TdpClearRectangle clearRectangle2 = validMessage.clearRectangle2;
        Assert.assertEquals(clearRectangle2.getLeft(), clearOp1.getLeft());
        Assert.assertEquals(clearRectangle2.getTop(), clearOp1.getTop());
        Assert.assertEquals(clearRectangle2.getWidth(), clearOp1.getWidth());
        Assert.assertEquals(clearRectangle2.getHeight(), clearOp1.getHeight());

        TdpLoadImage loadImg0 = (TdpLoadImage)actions.getOperations().get(step++);
        TdpLoadImage loadImage = validMessage.loadImage;
        Assert.assertEquals(loadImage.getLeft(), loadImg0.getLeft());
        Assert.assertEquals(loadImage.getTop(), loadImg0.getTop());
        Assert.assertEquals(loadImage.getImageType(), loadImg0.getImageType());
        Assert.assertEquals(loadImage.getImage().length, loadImg0.getImage().length);
        Assert.assertEquals(loadImage.getImage()[0], loadImg0.getImage()[0]);
        Assert.assertEquals(loadImage.getImage()[1], loadImg0.getImage()[1]);
        Assert.assertEquals(loadImage.getImage()[2], loadImg0.getImage()[2]);
        Assert.assertEquals(loadImage.getImage()[3], loadImg0.getImage()[3]);

        TdpLoadImage loadImg1 = (TdpLoadImage)actions.getOperations().get(step++);
        TdpLoadImage loadImage2 = validMessage.loadImage2;
        Assert.assertEquals(loadImage2.getLeft(), loadImg1.getLeft());
        Assert.assertEquals(loadImage2.getTop(), loadImg1.getTop());
        Assert.assertEquals(loadImage2.getImageType(), loadImg1.getImageType());
        Assert.assertEquals(loadImage2.getImage().length, loadImg1.getImage().length);
        Assert.assertEquals(loadImage2.getImage()[0], loadImg1.getImage()[0]);
        Assert.assertEquals(loadImage2.getImage()[1], loadImg1.getImage()[1]);
        Assert.assertEquals(loadImage2.getImage()[2], loadImg1.getImage()[2]);
        Assert.assertEquals(loadImage2.getImage()[3], loadImg1.getImage()[3]);

        TdpCopyImage copyImg1 = (TdpCopyImage)actions.getOperations().get(step++);
        TdpCopyImage copyImage = validMessage.copyImage;
        Assert.assertEquals(copyImage.getLeft(), copyImg1.getLeft());
        Assert.assertEquals(copyImage.getTop(), copyImg1.getTop());
        Assert.assertEquals(copyImage.getMemorySlot(), copyImg1.getMemorySlot());

        TdpCopyImage copyImg2 = (TdpCopyImage)actions.getOperations().get(step++);
        TdpCopyImage copyImage2 = validMessage.copyImage2;
        Assert.assertEquals(copyImage2.getLeft(), copyImg2.getLeft());
        Assert.assertEquals(copyImage2.getTop(), copyImg2.getTop());
        Assert.assertEquals(copyImage2.getMemorySlot(), copyImg2.getMemorySlot());

        TdpStoreImage store1 = (TdpStoreImage)actions.getOperations().get(step++);
        TdpStoreImage storeImage = validMessage.storeImage;
        Assert.assertEquals(storeImage.getMemorySlot(), store1.getMemorySlot());
    }

    /**
     * Tests the function parseData(): invalid tag
     */
    @Test
    public void testParseDataException1() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Unknown tag in data of ManipulateMemorySlot message"));

        // incorrect message (first tag id is invalid)
        final ByteBuffer data = TdpUtils.createBuffer(4);
        data.put((byte)0x4F);

        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        message.setData(data);

        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests the function parseData(): mandatory tag for InitializeWorkingMemory missing
     */
    @Test
    public void testParseDataException2() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Mandatory tag InitializeWorkingMemory missing"));

        // incorrect message (first tag id is invalid)
        final ByteBuffer data = TdpUtils.createBuffer(6);

        data.put((byte)0xC1); // clear rectangle tag
        data.put((byte)4);
        data.put(VlqUtils.getVlqBytes(10));
        data.put(VlqUtils.getVlqBytes(11));
        data.put(VlqUtils.getVlqBytes(12));
        data.put(VlqUtils.getVlqBytes(13));

        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        message.setData(data);

        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests the function parseData(): invalid message type
     */
    @Test
    public void testParseDataException3() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Wrong message type"));

        // incorrect message (message type is invalid)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.RESPONSE);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);

        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests the function parseData(): wrong command id
     */
    @Test
    public void testParseDataException4() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Wrong command id"));

        // incorrect message (command is invalid)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.STATUS);
        message.addAddress(1);
        message.addAddress(2);

        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests the function parseData(): Unknown image type in action LoadImage
     */
    @Test
    public void testParseDataException5() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Unknown value for ImageType"));

        final TdpLoadImage loadImage = new TdpLoadImage();
        loadImage.setLeft(127);
        loadImage.setTop(128);
        loadImage.setImageType(ETdpImageType.BMP);
        loadImage.setImage(new byte[] { 0, 1, 2, 3 });

        final ByteBuffer data = TdpUtils.createBuffer(13);
        data.put((byte)0xC0); // initialize working memory tag
        data.put((byte)2);
        data.put(VlqUtils.getVlqBytes(100));
        data.put(VlqUtils.getVlqBytes(100));
        data.put((byte)0xC2); // load image tag
        data.put((byte)7);
        data.put(VlqUtils.getVlqBytes(1));
        data.put(VlqUtils.getVlqBytes(1));
        data.put((byte)3);
        data.put(loadImage.getImage());

        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        message.setData(data);

        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests exceptions of the function parse(): no data
     */
    @Test
    public void testParseException6() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("No data in message"));

        // incorrect message (no data)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        message.setData(null);
        ManipulateMemorySlotCommand.parse(message);
    }

    /**
     * Tests the function CreateData() with maximum values
     */
    @Test
    public void testCreateDataMaximum() {
        TdpManipulateMemorySlot manipulateMemorySlot = new TdpManipulateMemorySlot();
        List<IImageOperation> operations = new ArrayList<IImageOperation>();
        operations.add(new TdpInitializeWorkingMemory(VlqUtils.MAX_VALUE, VlqUtils.MAX_VALUE));
        operations.add(new TdpClearRectangle(VlqUtils.MAX_VALUE, VlqUtils.MAX_VALUE, 
                VlqUtils.MAX_VALUE, VlqUtils.MAX_VALUE));
        operations.add(new TdpCopyImage(VlqUtils.MAX_VALUE, VlqUtils.MAX_VALUE, VlqUtils.MAX_VALUE));
        operations.add(new TdpLoadImage(ETdpImageType.PNG, new byte[] {0x7f, 0x7f}));
        operations.add(new TdpStoreImage(VlqUtils.MAX_VALUE));
        manipulateMemorySlot.setOperations(operations);
        ManipulateMemorySlotCommand.createData(manipulateMemorySlot);
    }

    private static final class ValidMessage {

        final TdpInitializeWorkingMemory initializeWorkingMemory;
        final TdpClearRectangle clearRectangle;
        final TdpClearRectangle clearRectangle2;
        final TdpLoadImage loadImage;
        final TdpLoadImage loadImage2;
        final TdpCopyImage copyImage;
        final TdpCopyImage copyImage2;
        final TdpStoreImage storeImage;

        public ValidMessage() {
            initializeWorkingMemory = new TdpInitializeWorkingMemory();
            initializeWorkingMemory.setWidth(128);
            initializeWorkingMemory.setHeight(127);

            clearRectangle = new TdpClearRectangle();
            clearRectangle.setLeft(127);
            clearRectangle.setTop(128);
            clearRectangle.setWidth(16383);
            clearRectangle.setHeight(16384);

            clearRectangle2 = new TdpClearRectangle();
            clearRectangle2.setLeft(0);
            clearRectangle2.setTop(0);
            clearRectangle2.setWidth(0);
            clearRectangle2.setHeight(0);

            loadImage = new TdpLoadImage();
            loadImage.setLeft(127);
            loadImage.setTop(128);
            loadImage.setImageType(ETdpImageType.BMP);
            loadImage.setImage(new byte[] { 0, 1, 2, 3 });
            
            loadImage2 = new TdpLoadImage();
            loadImage2.setLeft(0);
            loadImage2.setTop(0);
            loadImage2.setImageType(ETdpImageType.PNG);
            loadImage2.setImage(new byte[] { (byte)0xFF, (byte)0xFF, (byte)0xFF, 0x00 });

            copyImage = new TdpCopyImage();
            copyImage.setLeft(127);
            copyImage.setTop(128);
            copyImage.setMemorySlot(128);

            copyImage2 = new TdpCopyImage();
            copyImage2.setLeft(0);
            copyImage2.setTop(0);
            copyImage2.setMemorySlot(0);

            storeImage = new TdpStoreImage();
            storeImage.setMemorySlot(1);
        }

        public TdpMessage buildTdpMessage() {
            final ByteBuffer data = TdpUtils.createBuffer(54);
            data.put((byte)0xC0); // initialize working memory tag
            data.put((byte)3);
            data.put(VlqUtils.getVlqBytes(initializeWorkingMemory.getWidth()));
            data.put(VlqUtils.getVlqBytes(initializeWorkingMemory.getHeight()));
            data.put((byte)0xC1); // clear rectangle tag
            data.put((byte)8);
            data.put(VlqUtils.getVlqBytes(clearRectangle.getLeft()));
            data.put(VlqUtils.getVlqBytes(clearRectangle.getTop()));
            data.put(VlqUtils.getVlqBytes(clearRectangle.getWidth()));
            data.put(VlqUtils.getVlqBytes(clearRectangle.getHeight()));
            data.put((byte)0xC1); // clear rectangle tag
            data.put((byte)4);
            data.put(VlqUtils.getVlqBytes(clearRectangle2.getLeft()));
            data.put(VlqUtils.getVlqBytes(clearRectangle2.getTop()));
            data.put(VlqUtils.getVlqBytes(clearRectangle2.getWidth()));
            data.put(VlqUtils.getVlqBytes(clearRectangle2.getHeight()));
            data.put((byte)0xC2); // load image tag
            data.put((byte)8);
            data.put(VlqUtils.getVlqBytes(loadImage.getLeft()));
            data.put(VlqUtils.getVlqBytes(loadImage.getTop()));
            data.put((byte)loadImage.getImageType().getValue());
            data.put(loadImage.getImage());
            data.put((byte)0xC2); // load image tag
            data.put((byte)7);
            data.put(VlqUtils.getVlqBytes(loadImage2.getLeft()));
            data.put(VlqUtils.getVlqBytes(loadImage2.getTop()));
            data.put((byte)loadImage2.getImageType().getValue());
            data.put(loadImage2.getImage());
            data.put((byte)0xC3); // copy image tag
            data.put((byte)5);
            data.put(VlqUtils.getVlqBytes(copyImage.getLeft()));
            data.put(VlqUtils.getVlqBytes(copyImage.getTop()));
            data.put(VlqUtils.getVlqBytes(copyImage.getMemorySlot()));
            data.put((byte)0xC3); // copy image tag
            data.put((byte)3);
            data.put(VlqUtils.getVlqBytes(copyImage2.getLeft()));
            data.put(VlqUtils.getVlqBytes(copyImage2.getTop()));
            data.put(VlqUtils.getVlqBytes(copyImage2.getMemorySlot()));
            data.put((byte)0x44); // store image tag
            data.put(VlqUtils.getVlqBytes(storeImage.getMemorySlot()));

            final TdpMessage message = new TdpMessage();
            message.setMessageType(ETdpMessageType.COMMAND);
            message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
            message.addAddress(1);
            message.setData(data);
            return message;
        }
        
    }
}
