/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.messages;

import java.nio.ByteBuffer;

import org.disperanto.enums.ETdpCommand;
import org.disperanto.enums.ETdpMessageType;
import org.disperanto.exception.TdpProtocolException;
import org.disperanto.utils.TdpUtils;
import org.hamcrest.CoreMatchers;
import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * JUnit tests for class ManipulateMemorySlot.
 */
public class TdpResponseManipulateMemorySlotTest {

    @Rule
    public ExpectedException thrown = ExpectedException.none();

    /**
     * Tests the function parse()
     */
    @Test
    public final void testParse() {
        final short crc = (short)0xFFFF;

        try {
            final ByteBuffer data = TdpUtils.createBuffer(2);
            data.putShort(crc);

            // correct message
            final TdpMessage message = new TdpMessage();
            message.setMessageType(ETdpMessageType.RESPONSE);
            message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
            message.addAddress(1);
            message.setData(data);
            final int crcParsed = ManipulateMemorySlotResponse.parse(message);
            Assert.assertEquals(crc, crcParsed);
        } catch (TdpProtocolException e) {
            e.printStackTrace();
            Assert.fail("Unexpected TdpProtocolException");
        }
    }

    /**
     * Tests exceptions of the function parse(): Wrong response id
     */
    @Test
    public final void testParseException1() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Wrong command id in response"));

        // incorrect message (wrong response id)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.RESPONSE);
        message.setCommand(ETdpCommand.DISPLAY_PROPERTIES);
        message.addAddress(1);
        ManipulateMemorySlotResponse.parse(message);
    }

    /**
     * Tests exceptions of the function parse(): invalid message type
     */
    @Test
    public final void testParseException2() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("Wrong message type in response"));

        // incorrect message (response)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.COMMAND);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        ManipulateMemorySlotResponse.parse(message);
    }

    /**
     * Tests exceptions of the function parse(): no data
     */
    @Test
    public final void testParseException3() throws TdpProtocolException {
        thrown.expect(TdpProtocolException.class);
        thrown.expectMessage(CoreMatchers.containsString("No data in message"));

        // incorrect message (no data)
        final TdpMessage message = new TdpMessage();
        message.setMessageType(ETdpMessageType.RESPONSE);
        message.setCommand(ETdpCommand.MANIPULATE_MEMORY_SLOT);
        message.addAddress(1);
        message.setData(null);
        ManipulateMemorySlotResponse.parse(message);
    }

    /**
     * Tests the function createData() with maximum values
     */
    @Test
    public final void testCreateDataMaximum() {
        ManipulateMemorySlotResponse.createData((short)0xffff);
    }
}
