package org.disperanto.protocol;

import java.io.IOException;
import java.nio.ByteBuffer;

import org.disperanto.communication.LoopbackConnection;
import org.disperanto.messages.TestMessageLoader;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class TdpProtocolReceiverTest {

    private TdpProtocolReceiver receiver;
    private LoopbackConnection connection;

    private boolean connectionStopped;

    @Before
    public void before() throws IOException {
        connection = new TestConnection();

        receiver = new TdpProtocolReceiver(connection);

        connection.start();
    }

    @After
    public void after() throws InterruptedException {
        receiver.stop();
    }

    @Test
    public void handleProtocolException() throws IOException {
        byte[] msg = TestMessageLoader.loadMessageResource(getClass(), "invalid-crc.bin");
        connection.send(ByteBuffer.wrap(msg));
        Assert.assertEquals(false, connectionStopped);
        receiver.run();
        Assert.assertEquals(true, connectionStopped);
    }

    private class TestConnection extends LoopbackConnection {

        @Override
        public void start() throws IOException {
            super.start();

            connectionStopped = false;
        }

        @Override
        public void stop() {
            super.stop();

            connectionStopped = true;
            receiver.cancel(); // Prevent restart behavior
        }
    }

}
