/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.utils;

import java.nio.ByteBuffer;

import org.disperanto.exception.TdpProtocolException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * JUnit tests for class CompactTlvReader.
 */
public class CompactTlvReaderTest {
    private CompactTlvReader reader;

    /**
     * Default setup for each test.
     */
    @Before
    public final void setUp() {
        reader = new CompactTlvReader(ByteBuffer.allocate(1));
    }

    /**
     * Test the function getDataLength()
     */
    @Test
    public final void testGetDataLength() {
        byte tag;
        int length;

        try {
            // no data bytes (0 in bit 0-5)
            tag = (byte)0x00;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(0, length);

            // no data bytes (1 in bit 0-5)
            tag = (byte)0x3F;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(0, length);

            // 1 data byte (0 in bit 0-5)
            tag = (byte)0x40;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(1, length);

            // 1 data byte (1 in bit 0-5)
            tag = (byte)0x7F;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(1, length);

            // 2 data bytes (0 in bit 0-5)
            tag = (byte)0x80;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(2, length);

            // 2 data bytes (1 in bit 0-5)
            tag = (byte)0xBF;
            length = reader.getTlvDataLength(tag, null);
            Assert.assertEquals(2, length);

            // VLQ data bytes used (0 in bit 0-5)
            tag = (byte)0xC0;
            final ByteBuffer vlq = ByteBuffer.allocate(1);
            vlq.put((byte)0x04);
            vlq.rewind();
            length = reader.getTlvDataLength(tag, vlq);
            Assert.assertEquals(4, length);

            // VLQ data bytes used (1 in bit 0-5)
            tag = (byte)0xFF;
            final ByteBuffer vlq2 = ByteBuffer.allocate(2);
            vlq2.put((byte)0x81);
            vlq2.put((byte)0x00);
            vlq2.rewind();
            length = reader.getTlvDataLength(tag, vlq2);
            Assert.assertEquals(128, length);

        } catch (TdpProtocolException e) {
            // should not happen
            Assert.fail("Unexpected TdpProtocolException");
        }
    }
}
