/*
Copyright (c) 2013, Technolution B.V.
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this 
   list of conditions and the following disclaimer in the documentation and/or 
   other materials provided with the distribution.

3. Neither the name of Technolution nor the names of its contributors may be used 
   to endorse or promote products derived from this software without specific 
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/
package org.disperanto.utils;

import java.nio.ByteBuffer;

import org.disperanto.exception.TdpProtocolException;
import org.junit.Assert;
import org.junit.Test;

/**
 * JUnit tests for class VlqUtils.
 */
public class VlqUtilsTest {

    /**
     * Test the function countBits()
     */
    @Test
    public final void testCountBits() {
        int count = VlqUtils.countVlqBytes(0);
        Assert.assertEquals(1, count);

        count = VlqUtils.countVlqBytes(127);
        Assert.assertEquals(1, count);

        count = VlqUtils.countVlqBytes(128);
        Assert.assertEquals(2, count);

        count = VlqUtils.countVlqBytes(16383);
        Assert.assertEquals(2, count);

        count = VlqUtils.countVlqBytes(16384);
        Assert.assertEquals(3, count);

        count = VlqUtils.countVlqBytes(2097151);
        Assert.assertEquals(3, count);

        count = VlqUtils.countVlqBytes(2097152);
        Assert.assertEquals(4, count);

        count = VlqUtils.countVlqBytes(268435455);
        Assert.assertEquals(4, count);

        count = VlqUtils.countVlqBytes(268435456);
        Assert.assertEquals(5, count);

        count = VlqUtils.countVlqBytes(VlqUtils.MAX_VALUE);
        Assert.assertEquals(VlqUtils.MAX_VLQ_BYTES, count);
    }

    /**
     * Tests the function getVlqBytes()
     */
    @Test
    public final void testGetVlqBytes() {
        int dataLength = 127;
        ByteBuffer buffer = ByteBuffer.allocate(VlqUtils.countVlqBytes(dataLength));
        buffer = VlqUtils.getVlqBytes(dataLength);
        buffer.rewind();
        Assert.assertEquals((byte)0x7F, buffer.get());

        dataLength = 128;
        buffer = ByteBuffer.allocate(VlqUtils.countVlqBytes(dataLength));
        buffer = VlqUtils.getVlqBytes(dataLength);
        buffer.rewind();
        Assert.assertEquals((byte)0x81, buffer.get());
        Assert.assertEquals((byte)0x00, buffer.get());
    }

    /**
     * Tests the function parseVlqData()
     */
    @Test
    public final void testParseVlqBytes() {
        try {
            ByteBuffer buffer = ByteBuffer.allocate(1);
            buffer.put((byte)0x7F);
            buffer.rewind();
            int dataLength = VlqUtils.parseVlqData(buffer);
            Assert.assertEquals(127, dataLength);

            buffer = ByteBuffer.allocate(2);
            buffer.put((byte)0x81);
            buffer.put((byte)0x00);
            buffer.rewind();
            dataLength = VlqUtils.parseVlqData(buffer);
            Assert.assertEquals(128, dataLength);

            buffer = ByteBuffer.allocate(2);
            buffer.put((byte)0xFF);
            buffer.put((byte)0x7F);
            buffer.rewind();
            dataLength = VlqUtils.parseVlqData(buffer);
            Assert.assertEquals(16383, dataLength);

            buffer = ByteBuffer.allocate(3);
            buffer.put((byte)0xFF);
            buffer.put((byte)0xFF);
            buffer.put((byte)0x7F);
            buffer.rewind();
            dataLength = VlqUtils.parseVlqData(buffer);
            Assert.assertEquals(2097151, dataLength);

            buffer = ByteBuffer.allocate(4);
            buffer.put((byte)0xFF);
            buffer.put((byte)0xFF);
            buffer.put((byte)0xFF);
            buffer.put((byte)0x7F);
            buffer.rewind();
            dataLength = VlqUtils.parseVlqData(buffer);
            Assert.assertEquals(268435455, dataLength);

        } catch (TdpProtocolException e) {
            e.printStackTrace();
            Assert.fail("Unexpected TdpProtocolException");
        }
    }

    /**
     * Tests the function parseVlqData(): not enough data bytes
     */
    @Test(expected = TdpProtocolException.class)
    public final void testParseVlqBytesException() throws TdpProtocolException {
        final ByteBuffer buffer = ByteBuffer.allocate(2);
        buffer.put((byte)0x81);
        buffer.put((byte)0x80);
        buffer.rewind();
        VlqUtils.parseVlqData(buffer);
    }
}
